#! /bin/bash
#
# xawdecode_record.sh
#  
# Made by (Cyriac BENOIT)
# Login   <cyriac@free-unices.org>
#
# Started on  Mon Sep 22 01:03:19 2003 Cyriac BENOIT
# Last update Thu Oct  2 02:33:32 2003 Cyriac BENOIT
#

# The only parameters to be configured...
config_file="${HOME}/.xawdecode/xawdecoderc"
config_dir="${HOME}/.xawdecode/records"

# You shouldn't edit no further
VERSION="0.0.9"
channel=""
start_time=""
end_time=""
avi_dir="${HOME}"
avi_title=".avi"
channel_number=""
expert="OFF"
quick="OFF"

if [ ! -d $config_dir ]; then
  mkdir $config_dir
fi

# Helper function
usage () {
  echo "Syntax: $(basename $0) [-c channel] [-d date ] [-s start time]"
  echo "        [-e end time] [-D avi directory] [-t avi title] [-x | --expert]"
  echo "        [-q | --quick] [-h | --help]"
  echo ""
  echo "        Date is in DD.MM.YY format and times are in HH:MM format."
  echo ""
  echo "        -x will enable \"expert mode\", asking more advanced parameters."
  echo "        -q will enable \"quick mode\", asking only date, times and channel"
  echo "           (if not given on command line)."
  echo ""
  exit 1
}

# Time functions
get_time () {
  echo -n "Enter $1 time (HH:MM): "
  read $2
}

get_recording_time () {
  start_hour=${1::2}
  start_minute=${1:3}
  stop_hour=${2::2}
  stop_minute=${2:3}

  start=$((start_hour * 60 + start_minute))
  stop=$((stop_hour * 60 + stop_minute))

  # Adding a whole day for special cases (23-30 - 01:15)
  if [ $stop -lt $start ]; then
    stop=$((stop + 24 * 60))
  fi

  record_time=$((stop-start))
  echo "Recording time will be $record_time minutes"
  record_time=$((record_time * 60))
}

# Channel functions
get_channel () {

  # Parsing config file, looking for channel names ([name]) and stripping
  # square brackets
  channames=$(grep "^\[" $config_file | tr -d "\[\]")

  # Setting PS3 prompt (for select) and IFS as newline
  PS3="Select channel: "
  oldifs="$IFS"
  IFS=$'\n'
  select channel in $channames; do
    echo $channel
    break
  done
  channel_number=$((REPLY - 1))

  # Restoring IFS
  IFS="$oldifs"
}

get_channel_name () {
  channel=$1
  grep "^\[" $config_file | awk 'NR == '$channel' { print }' | tr -d "\[\]"
}

# Menu functions

# Delete
delete_job () {
  echo -n "Do you want to delete recording job $1 [N/y] ? "
  read answer
  if [ "$answer" = "y" -o "$answer" = "Y" ]; then

    # Removing script file
    script=$(grep -H "###JOB### $job_to_delete" $config_dir/* | cut -d':' -f1)
    rm -f $script

    # Call to atrm
    atrm $1
    echo "Job deleted. Press Enter..."
    read
  fi
}

# List
list_records () {
  clear
  if [ $(ls $config_dir | wc -l) -ne 0 ]; then

    # We get all informations from script files
    for script in $config_dir/*; do
      infos=$(grep "###INFOS###" $script)
      date=$(echo $infos | awk '{ print $2 }')
      year=$(echo $date | cut -c 1-4)
      month=$(echo $date | cut -c 5-6)
      day=$(echo $date | cut -c 7-8)
      date="${day}.${month}.${year}"
      channel=$(echo $infos | awk '{ print $3 }')
      channel=$(get_channel_name $((channel + 1)))
      time=$(echo $infos | awk '{ print $4 }')
      job=$(grep "###JOB###" $script | awk '{ print $2 }')
      echo "$job) $date on $channel at $time"
    done
  else

    # Or not
    echo "No records scheduled."
  fi
  echo ""
  echo -n "$@"
  read job_to_delete
  channel=""
  return $job_to_delete
}

# Create xawdecode parameters line
make_command_line () {

  # Forcing grabdisplay in order to record something...
  parameters="-m -capture grabdisplay"
  
  if [ -n "$codec" ]; then
    parameters="${parameters} -rec_vcodec $codec"
  fi

  if [ -n "$size" ]; then
    parameters="${parameters} -rec_width $width -rec_height $height"
  fi

  if [ -n "$sub" ]; then
    parameters="${parameters} -rec_sub 1 -novbi"
  fi

  if [ -n "$quality" ]; then
    parameters="${parameters} -rec_mp3vbr $quality"
  fi

  if [ -n "$bitrate" ]; then
    parameters="${parameters} -rec_mp3cbr $bitrate"
  fi

  if [ -n "$nbufs" ]; then
    parameters="${parameters} -nbufs $nbufs"
  fi
}

# Options parsing
while getopts ":c:s:e:d:D:t:qhvx-:" option; do
  if [ "$option" = "-" ]; then
    case $OPTARG in
      help) option=h ;;
      quick) option=q ;;
      expert) option=x ;;
      version) option=v ;;
    esac
  fi
  case $option in
    c) channel=$OPTARG ;;
    s) start_time=$OPTARG ;;
    e) end_time=$OPTARG ;;
    d) rec_date=$OPTARG ;;
    D) avi_dir=$OPTARG ;;
    t) avi_title=$OPTARG ;;
    x) expert="ON" ;;
    q) quick="ON" ;;
    h) usage ;;
    v) echo "$(basename $0) version $VERSION"
       exit 0
       ;;
    :) echo "Missing argument for option -$OPTARG"
       usage ;;
    ?) echo "Unknown option -$OPTARG"
       usage ;;
  esac
done

# Main menu
while [ 1 -a "$quick" = "OFF" ]; do
  clear
  echo "1) Schedule a record"
  echo "2) Schedule a record in expert mode"
  echo "3) List scheduled records"
  echo "4) Delete a recording job"
  echo "5) Quit"
  echo ""
  echo -n "Enter your choice [1]: "

  read choice
  case "$choice" in
    2) expert="ON"
       clear
       break ;;
    3) list_records "Press Enter..." ;;
    4) list_records "Enter job to delete: "
       if [ -n "$job_to_delete" ]; then
         delete_job $job_to_delete
       fi
       ;;
    5) exit 0 ;;
    *) clear; break ;;
  esac
done

# Getting day
if [ -z "$rec_date" ]; then
  echo -n "Enter recording date (DD.MM.YY) [$(date +"%d.%m.%y")]: "
  read rec_date
  if [ -z "$rec_date" ]; then
    rec_date=$(date +"%d.%m.%y")
  fi
fi

# Getting missing hours
if [ -z "$start_time" ]; then
  get_time start start_time
fi

if [ -z "$end_time" ]; then
  get_time end end_time
fi

# Display total recording time
get_recording_time $start_time $end_time
echo ""

# Getting channel name and number
if [ -z "$channel" ]; then
  get_channel
else 
  channel_number=$((channel - 1))
  channel=$(get_channel_name $channel)
fi

echo ""

# Getting AVI directory
if [ "$avi_dir" = "${HOME}" -a "$quick" = "OFF" ]; then
  echo -n "Enter saving directory [${HOME}]: "
  read avi_dir
fi

if [ ! -d "$avi_dir" -o -z "$avi_dir" ]; then
  avi_dir=${HOME}
fi

# Getting AVI title
if [ "$avi_title" = ".avi" -a "$quick" = "OFF" ]; then
  echo -n "Enter movie title [${channel}-${start_time}-${end_time}]: "
  read avi_title
  avi_title="${avi_title}.avi"
fi

if [ "$avi_title" = ".avi" ]; then
  avi_title="${channel}-${start_time}-${end_time}.avi"
fi

echo ""

# If we in expert mode, we ask more questions
if [ "$expert" = "ON" ]; then

  # Getting codec - We do not verify that all codecs are supported !!
  codecs=(xvid divx4linux ffmpeg_mpeg1 ffmpeg_mpeg4)
  i=1
  for name in ${codecs[*]}; do
    echo "$i) $name"
    i=$((i + 1))
  done
  echo -n "Enter codec name [ffmpeg_mpeg4]: "
  read codec_number

  if [ -z "$codec_number" ]; then
    codec="ffmpeg_mpeg4"
  else
    codec=${codecs[$codec_number - 1]}
  fi

  echo ""

  # Getting recording size
  sizes=(384x288 480x360 576x432 768x576)
  i=1
  for size in ${sizes[*]}; do
    echo "$i) $size"
    i=$((i + 1))
  done
  echo "$i) Other"
  echo -n "Enter recording size [384x288]: "
  read size

  if [ -z "$size" ]; then
    size="384x288"
  elif [ $size -eq $i ]; then
    echo -n "Enter your choice: "
    read size
  else
    size=${sizes[$size - 1]}
  fi
  
  # Get width and height from size
  width=${size/x*}
  height=${size/#*x}

  echo ""
  echo -n "Do you wanna record subtitles [N/y] ? "
  read sub
  
  echo ""
  echo -n "Modify sound parameters [N/y] ? "
  read answer
  if [ "$answer" = "y" -o "$answer" = "Y" ]; then
    echo -n "Constant or variable bitrate [C/v] ?"
    read answer
    if [ "$answer" = "v" -o "$answer" = "V" ]; then
      echo -n "Enter quality (0=best, 9=worst) [8]: "
      read quality
      if [ -z "$quality" ]; then
        quality=8
      fi
    else
      echo -n "Enter bitrate (16-512) [128]: "
      read bitrate
      if [ -z "bitrate" ]; then
        bitrate=128
      fi
    fi
  fi

  echo ""
  echo -n "Enter number of kernel-buffered frames [3]: "
  read nbufs
  if [ -z "$nbufs" ]; then
    nbufs=3
  fi

  echo ""
  echo -n "Do you wanna mute sound (for night recording) [Y/n] ?"
  read sound

# End of expert mode
fi

make_command_line

# Creating script name
#if [ $start_hour -lt $(date +"%H") ]; then
#  date=$(date -d tomorrow +"%Y%m%d")
#else
#  date=$(date +"%Y%m%d")
#fi
date=$(echo $rec_date | sed -e 's/\(..\).\(..\).\(..\)/20\3\2\1/')
file="$config_dir/${date}_${channel_number}_${start_time/:/}"

# We need Xawdecode filename to rename it
default_avi="movie-${channel// /\\ }-${date}-${start_time/:/}??-1.avi"

# Creating script
cat >> $file <<_END_

###INFOS### $date $channel_number $start_time

export DISPLAY=:0
cd $avi_dir
_END_

if [ -z "$sound" -o "$sound" = "y" -o "$sound" = "Y" ]; then
  echo "amixer set Master mute >/dev/null 2>&1" >> $file
fi

cat >> $file <<_END_
echo "Will kill all running xawdecode !"
killall xawdecode
xawdecode $parameters -e "setstation $channel_number; record" &
sleep $record_time
xawdecode_cmd "record; quit"
sleep 1
amixer set Line mute >/dev/null 2>&1
mv $default_avi "${avi_title}"
_END_

if [ -z "$sound" -o "$sound" = "y" -o "$sound" = "Y" ]; then
  echo "amixer set Master unmute >/dev/null 2>&1" >> $file
fi

echo "rm -f $file" >> $file

# Scheduling
job=$(at -m -f $file $start_time $rec_date 2>&1 | grep job | cut -d' ' -f2)
echo -e "\n###JOB### $job\n" >> $file

# Display informations
echo "Will record $avi_title on channel $channel_number ($channel) from $start_time to $end_time in $avi_dir, the $rec_date"
echo "Script in $file"


